@@ -199,10 +199,7 @@ protect_argv (gchar  **argv,
 GQuark
 g_spawn_error_quark (void)
 {
-  static GQuark quark = 0;
-  if (quark == 0)
-    quark = g_quark_from_static_string ("g-exec-error-quark");
-  return quark;
+  return g_quark_from_static_string ("g-exec-error-quark");
 }
 
 gboolean
@@ -1038,9 +1035,12 @@ g_spawn_sync_utf8 (const gchar          *working_directory,
       outstr = g_string_new (NULL);
       outchannel = g_io_channel_win32_new_fd (outpipe);
       g_io_channel_set_encoding (outchannel, NULL, NULL);
+      g_io_channel_set_buffered (outchannel, FALSE);
       g_io_channel_win32_make_pollfd (outchannel,
 				      G_IO_IN | G_IO_ERR | G_IO_HUP,
 				      &outfd);
+      if (debug)
+	g_print ("outfd=%x\n", outfd.fd);
     }
       
   if (errpipe >= 0)
@@ -1048,9 +1048,12 @@ g_spawn_sync_utf8 (const gchar          *working_directory,
       errstr = g_string_new (NULL);
       errchannel = g_io_channel_win32_new_fd (errpipe);
       g_io_channel_set_encoding (errchannel, NULL, NULL);
+      g_io_channel_set_buffered (errchannel, FALSE);
       g_io_channel_win32_make_pollfd (errchannel,
 				      G_IO_IN | G_IO_ERR | G_IO_HUP,
 				      &errfd);
+      if (debug)
+	g_print ("errfd=%x\n", errfd.fd);
     }
 
   /* Read data until we get EOF on all pipes. */
