@@ -198,6 +198,49 @@ check_integer_list_value (GKeyFile    *keyfile,
   g_free (value);
 }
 
+static void
+check_double_list_value (GKeyFile    *keyfile,
+			  const gchar *group,
+			  const gchar *key,
+			  ...)
+{
+  gint i;
+  gdouble v, *value;
+  va_list args;
+  gsize len;
+  GError *error = NULL;
+
+  value = g_key_file_get_double_list (keyfile, group, key, &len, &error);
+  check_no_error (&error);
+  g_assert (value != NULL);
+  
+  va_start (args, key);
+  i = 0;
+  v = va_arg (args, gdouble);
+  while (v != -100)
+    {
+      if (i == len)
+	{
+	  g_print ("Group %s key %s: list too short (%d)\n", 
+		   group, key, i);      
+	  exit (1);
+	}
+      if (value[i] != v)
+	{
+	  g_print ("Group %s key %s: mismatch at %d, expected %e, got %e\n", 
+		   group, key, i, v, value[i]);      
+	  exit (1);
+	}
+
+      i++;
+      v = va_arg (args, gdouble);
+    }
+
+  va_end (args);
+  
+  g_free (value);
+}
+
 static void
 check_boolean_list_value (GKeyFile    *keyfile,
 			  const gchar *group,
@@ -285,15 +328,36 @@ check_integer_value (GKeyFile    *keyfile,
     }
 }
 
+static void
+check_double_value (GKeyFile    *keyfile,
+		     const gchar *group,
+		     const gchar *key,
+		     gdouble      expected) 
+{
+  GError *error = NULL;
+  gdouble value;
+
+  value = g_key_file_get_double (keyfile, group, key, &error);
+  check_no_error (&error);
+
+  if (value != expected)
+    {
+      g_print ("Group %s key %s: "
+	       "expected integer value %e, actual value %e\n",
+	       group, key, expected, value);      
+      exit (1);
+    }
+}
+
 static void
 check_name (const gchar *what,
 	    const gchar *value,
 	    const gchar *expected,
 	    gint         position)
 {
-  if (strcmp (expected, value) != 0)
+  if (!value || strcmp (expected, value) != 0)
     {
-      g_print ("Wrong %s returned: got %s at %d, expected %s\n",
+      g_print ("Wrong %s returned: got '%s' at %d, expected '%s'\n",
 	       what, value, position, expected);
       exit (1);
     }
@@ -367,6 +431,94 @@ test_whitespace (void)
   g_key_file_free (keyfile);
 }
 
+/* check handling of comments
+ */
+static void
+test_comments (void)
+{
+  GKeyFile *keyfile;
+  gchar **names;
+  gsize len;
+  GError *error = NULL;
+  gchar *comment;
+
+  const gchar *data = 
+    "# top comment\n"
+    "# top comment, continued\n"
+    "[group1]\n"
+    "key1 = value1\n"
+    "# key comment\n"
+    "# key comment, continued\n"
+    "key2 = value2\n"
+    "# line end check\r\n"
+    "key3 = value3\n"
+    "key4 = value4\n"
+    "# group comment\n"
+    "# group comment, continued\n"
+    "[group2]\n";
+
+  const gchar *top_comment= " top comment\n top comment, continued\n";
+  const gchar *group_comment= " group comment\n group comment, continued\n";
+  const gchar *key_comment= " key comment\n key comment, continued\n";
+  
+  keyfile = load_data (data, 0);
+
+  check_string_value (keyfile, "group1", "key1", "value1");
+  check_string_value (keyfile, "group1", "key2", "value2");
+  check_string_value (keyfile, "group1", "key3", "value3");
+  check_string_value (keyfile, "group1", "key4", "value4");
+
+  names = g_key_file_get_keys (keyfile, "group1", &len, &error);
+  check_no_error (&error);
+
+  check_length ("keys", g_strv_length (names), len, 4);
+  check_name ("key", names[0], "key1", 0);
+  check_name ("key", names[1], "key2", 1);
+  check_name ("key", names[2], "key3", 2);
+  check_name ("key", names[3], "key4", 3);
+
+  g_strfreev (names);
+
+  g_key_file_free (keyfile);
+
+  keyfile = load_data (data, G_KEY_FILE_KEEP_COMMENTS);
+
+  names = g_key_file_get_keys (keyfile, "group1", &len, &error);
+  check_no_error (&error);
+
+  check_length ("keys", g_strv_length (names), len, 4);
+  check_name ("key", names[0], "key1", 0);
+  check_name ("key", names[1], "key2", 1);
+  check_name ("key", names[2], "key3", 2);
+  check_name ("key", names[3], "key4", 3);
+
+  g_strfreev (names);
+
+  comment = g_key_file_get_comment (keyfile, NULL, NULL, &error);
+  check_no_error (&error);
+  check_name ("top comment", comment, top_comment, 0);
+  g_free (comment);
+
+  comment = g_key_file_get_comment (keyfile, "group1", "key2", &error);
+  check_no_error (&error);
+  check_name ("key comment", comment, key_comment, 0);
+  g_free (comment);
+
+  comment = g_key_file_get_comment (keyfile, "group2", NULL, &error);
+  check_no_error (&error);
+  check_name ("group comment", comment, group_comment, 0);
+  g_free (comment);
+
+  comment = g_key_file_get_comment (keyfile, "group3", NULL, &error);
+  check_error (&error, 
+	       G_KEY_FILE_ERROR,
+	       G_KEY_FILE_ERROR_GROUP_NOT_FOUND);
+  g_assert (comment == NULL);
+
+  g_key_file_free (keyfile);
+}
+
+
 /* check key and group listing */
 static void
 test_listing (void)
@@ -523,9 +675,9 @@ test_boolean (void)
   g_key_file_free (keyfile);
 }
 
-/* check parsing of integer values */
+/* check parsing of integer and double values */
 static void
-test_integer (void)
+test_number (void)
 {
   GKeyFile *keyfile;
   GError *error = NULL;
@@ -538,6 +690,9 @@ test_integer (void)
     "key4=2324431\n"
     "key5=-2324431\n"
     "key6=000111\n"
+    "dkey1=000111\n"
+    "dkey2=145.45\n"
+    "dkey3=-3453.7\n"
     "[invalid]\n"
     "key1=0xffff\n"
     "key2=0.5\n"
@@ -552,6 +707,9 @@ test_integer (void)
   check_integer_value (keyfile, "valid", "key4", 2324431);
   check_integer_value (keyfile, "valid", "key5", -2324431);
   check_integer_value (keyfile, "valid", "key6", 111);
+  check_double_value (keyfile, "valid", "dkey1", 111.0);
+  check_double_value (keyfile, "valid", "dkey2", 145.45);
+  check_double_value (keyfile, "valid", "dkey3", -3453.7);
 
   g_key_file_get_integer (keyfile, "invalid", "key1", &error);
   check_error (&error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
@@ -615,7 +773,7 @@ test_locale_string (void)
 
   /* now test that translations are thrown away */
 
-  g_setenv ("LC_ALL", "de", TRUE);
+  g_setenv ("LANGUAGE", "de", TRUE);
   setlocale (LC_ALL, "");
 
   keyfile = load_data (data, 0);
@@ -645,7 +803,8 @@ test_lists (void)
     "key5=true;false\n"
     "key6=1;0;-1\n"
     "key7= 1 ; 0 ; -1 \n"
-    "key8=v1\\,v2\n";
+    "key8=v1\\,v2\n"
+    "key9=0;1.3456;-76532.456\n";
   
   keyfile = load_data (data, 0);
 
@@ -655,6 +814,7 @@ test_lists (void)
   check_string_list_value (keyfile, "valid", "key4", "v1;v2", NULL);
   check_boolean_list_value (keyfile, "valid", "key5", TRUE, FALSE, -100);
   check_integer_list_value (keyfile, "valid", "key6", 1, 0, -1, -100);
+  check_double_list_value (keyfile, "valid", "key9", 0.0, 1.3456, -76532.456, -100.0);
   /* maybe these should be valid */
   /* check_integer_list_value (keyfile, "valid", "key7", 1, 0, -1, -100);*/
   /* check_string_list_value (keyfile, "valid", "key8", "v1\\,v2", NULL);*/
@@ -708,6 +868,8 @@ test_group_remove (void)
   g_key_file_remove_group (keyfile, "group1", &error);
   check_no_error (&error);
   
+  g_strfreev (names);
+
   names = g_key_file_get_groups (keyfile, &len);
   if (names == NULL)
     {
@@ -722,6 +884,8 @@ test_group_remove (void)
   g_key_file_remove_group (keyfile, "group2", &error);
   check_no_error (&error);
   
+  g_strfreev (names);
+
   names = g_key_file_get_groups (keyfile, &len);
   if (names == NULL)
     {
@@ -735,6 +899,8 @@ test_group_remove (void)
   g_key_file_remove_group (keyfile, "no such group", &error);
   check_error (&error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND);
 
+  g_strfreev (names);
+
   g_key_file_free (keyfile);
 }
 
@@ -772,19 +938,75 @@ test_key_remove (void)
 }
 
 
+/* http://bugzilla.gnome.org/show_bug.cgi?id=316309 */
+static void
+test_groups (void)
+{
+  GKeyFile *keyfile;
+
+  const gchar *data = 
+    "[1]\n"
+    "key1=123\n"
+    "[2]\n"
+    "key2=123\n";
+  
+  keyfile = load_data (data, 0);
+
+  check_string_value (keyfile, "1", "key1", "123");
+  check_string_value (keyfile, "2", "key2", "123");
+
+  g_key_file_free (keyfile);  
+}
+
+static void
+test_duplicate_keys (void)
+{
+  GKeyFile *keyfile;
+  const gchar *data = 
+    "[1]\n"
+    "key1=123\n"
+    "key1=345\n";
+
+  keyfile = load_data (data, 0);
+  check_string_value (keyfile, "1", "key1", "345");
+
+  g_key_file_free (keyfile);  
+}
+
+/* http://bugzilla.gnome.org/show_bug.cgi?id=157877 */
+static void
+test_duplicate_groups (void)
+{
+  GKeyFile *keyfile;
+  const gchar *data = 
+    "[Desktop Entry]\n"
+    "key1=123\n"
+    "[Desktop Entry]\n"
+    "key2=123\n";
+  
+  keyfile = load_data (data, 0);
+
+  g_key_file_free (keyfile);  
+}
+
+
 int
 main (int argc, char *argv[])
 {
   test_line_ends ();
   test_whitespace ();
+  test_comments ();
   test_listing ();
   test_string ();
   test_boolean ();
-  test_integer ();
+  test_number ();
   test_locale_string ();
   test_lists ();
   test_group_remove ();
   test_key_remove ();
+  test_groups ();
+  test_duplicate_keys ();
+  test_duplicate_groups ();
   
   return 0;
 }
